import json
import os.path

from django.forms.widgets import Media


PACKAGE_DIR = os.path.dirname(os.path.abspath(__file__))
CODEMIRROR_DIR = os.path.join(PACKAGE_DIR, 'static/django-mirror/codemirror')


MEDIA_PATHS = {}
with open(os.path.join(PACKAGE_DIR, 'media.json')) as f:
    MEDIA_PATHS = json.load(f)


def _follow(path):
    """
    Recursively gather the list of paths required by the given path.
    """
    reqs = []
    if path in MEDIA_PATHS:
        for another in MEDIA_PATHS[path]:
            reqs += _follow(another)
    return reqs + [path]


def _make(path):
    """
    Init a media instance for the given path.
    """
    css, js = [], []

    for req in _follow(path):
        req = os.path.join('django-mirror/codemirror', req)

        if req.endswith('.css'):
            css.append(req)
        elif req.endswith('.js'):
            js.append(req)

    return Media(css={'all': css}, js=js)


def get_mode_media(mode):
    """
    Return a media instance pointing to the css and js files required by the
    given mode. Raise an error if the mode cannot be recognised.
    """
    path = 'mode/{}/{}.js'.format(mode, mode)

    if path not in MEDIA_PATHS:
        raise ValueError('Unknown mode: {!s}'.format(mode))

    return _make(path)


def get_addon_media(addon):
    """
    Return a media instance pointing to the css and js files required by the
    given addon. Raise an error if the mode cannot be recognised.
    """
    path = 'addon/{}'.format(addon if addon.endswith('.js') else addon+'.js')

    if path not in MEDIA_PATHS:
        raise ValueError('Unknown addon: {!s}'.format(addon))

    return _make(path)


def get_theme_media(theme):
    """
    Return a media instance pointing to the css file of the requested theme.
    Raise an error if the theme cannot be recognised.
    """
    path = 'theme/{}.css'.format(theme)

    if not os.path.exists(os.path.join(CODEMIRROR_DIR, path)):
        raise ValueError('Unknown theme: {!s}'.format(theme))

    path = os.path.join('django-mirror/codemirror', path)

    return Media(css={'all': [path]})



from django_mirror.widgets import AdminMirrorArea


class MirrorAdmin:
    """
    Mixin for ModelAdmin subclasses which provides the mirror_fields option,
    used for listing fields to be rendered with the MirrorArea widget.

    mirror_fields can be either a list of field names, or a list of tuples
    where the first item is the field name and the second item is a dict of
    options to be passed to the widget.

    This mixin just provides another way to use the MirrorArea widget in the
    admin; this could be also done by subclassing the model form or by using
    the formfield_overrides option.
    """
    mirror_fields = tuple()

    def get_mirror_fields(self, request, obj=None):
        """
        Hook for specifying custom mirror fields.
        """
        return self.mirror_fields

    def _update_form_factory_kwargs(self, request, obj=None, **kwargs):
        """
        Update kwargs['widgets'] to include the mirror area widgets as defined
        in the mirror_fields option.

        Helper for the get_form and get_formset methods.
        """
        mirror_fields = self.get_mirror_fields(request, obj)
        mirror_widgets = {}

        for field in mirror_fields:
            if isinstance(field, (list, tuple)):
                field_name = field[0]
                widget_args = field[1]
            else:
                field_name = str(field)
                widget_args = {}

            mirror_widgets[field_name] = AdminMirrorArea(**widget_args)

        if mirror_widgets:
            if 'widgets' in kwargs and kwargs['widgets'] is not None:
                kwargs['widgets'].update(mirror_widgets)
            else:
                kwargs['widgets'] = mirror_widgets

        return kwargs

    def get_form(self, request, obj=None, **kwargs):
        """
        Overwrite ModelAdmin's get_form method in order to update the widgets
        argument for the modelform_factory.
        """
        kwargs = self._update_form_factory_kwargs(request, obj, **kwargs)
        return super().get_form(request, obj, **kwargs)

    def get_formset(self, request, obj=None, **kwargs):
        """
        Overwrite InlineModelAdmin's get_formset method in order to update the
        widgets argument for the inlineformset_factory.
        """
        kwargs = self._update_form_factory_kwargs(request, obj, **kwargs)
        return super().get_formset(request, obj, **kwargs)



import json
import logging

from django import forms

from django_mirror.config import Config
from django_mirror.media import (
    get_addon_media, get_mode_media, get_theme_media
)


logger = logging.getLogger(__name__)


class MirrorArea(forms.Textarea):
    """
    Form widget that renders as a standard <textarea> but also includes the
    relevant static files that convert it into a CodeMirror editor.
    """

    def __init__(self, attrs=None, addons=[], **kwargs):
        """
        Set the config options for the CodeMirror editor.
        """
        self.config = Config()
        self.config.update(**kwargs)

        if addons:
            self.config.set_addons(addons)

        if not attrs:
            attrs = {}
        attrs['data-mirror'] = json.dumps(self.config.options)

        super().__init__(attrs)

    @property
    def media(self):
        """
        Dynamically define the css and js assets needed by the widget.
        """
        media = forms.Media(
            css={
                'all': ['django-mirror/codemirror/lib/codemirror.css']
            },
            js=['django-mirror/codemirror/lib/codemirror.js']
        )

        if 'mode' in self.config.options:
            try:
                media += get_mode_media(self.config.options['mode'])
            except ValueError as error:
                logger.error(str(error))

        if 'theme' in self.config.options:
            try:
                media += get_theme_media(self.config.options['theme'])
            except ValueError as error:
                logger.error(str(error))

        for addon in self.config.addons:
            try:
                media += get_addon_media(addon)
            except ValueError as error:
                logger.error(str(error))

        media += forms.Media(js=['django-mirror/init.js'])

        return media


class AdminMirrorArea(MirrorArea):
    """
    The same widget in the context of the Django admin.
    """

    @property
    def media(self):
        """
        Include admin.css, which tries to make the CodeMirror editor instances
        look a bit more like regular admin textarea fields.
        """
        admin_media = forms.Media(
            css = {
                'all': ('django-mirror/admin.css',)
            }
        )
        return super().media + admin_media



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


def snake_to_camel(snake):
    """
    Convert a string from snake_case to camelCase.
    """
    first, *rest = snake.split('_')
    return first + ''.join(word.capitalize() for word in rest)


class Config:
    """
    A config instance represents a CodeMirror configuration [1]. Usage:

        config = Config()
        config.update(read_only=True, addons=['dialog/dialog'])
        assert config.options == {'readOnly': True}
        assert config.addons == ['dialog/dialog.js']

    [1]: https://codemirror.net/doc/manual.html#config
    """

    def __init__(self):
        """
        Init the config with the default values provided in the settings.
        """
        self.options = {}
        self.addons = []

        if hasattr(settings, 'DJANGO_MIRROR_DEFAULTS'):
            try:
                self.update(**settings.DJANGO_MIRROR_DEFAULTS)
            except TypeError:
                message = 'DJANGO_MIRROR_DEFAULTS should be a dict.'
                raise ImproperlyConfigured(message)

    def update(self, **kwargs):
        for key, value in kwargs.items():
            if key == 'addons':
                self.set_addons(value)
            else:
                self.options[snake_to_camel(key)] = value

    def set_addons(self, addons):
        self.addons = [
            addon if addon.endswith('.js') else addon + '.js'
            for addon in addons
        ]



import os.path

from setuptools import setup, find_packages


BASE_DIR = os.path.abspath(os.path.dirname(__file__))


with open(os.path.join(BASE_DIR, 'README.rst')) as f:
    README = f.read()


with open(os.path.join(BASE_DIR, 'django_mirror/__version__.py')) as f:
    exec(f.read())


setup(
    name='django-mirror',
    version=VERSION,

    description='This package marries Django with CodeMirror',
    long_description=README,
    long_description_content_type='text/x-rst',

    url='https://github.com/pavelsof/django-mirror',
    author='pavelsof',
    author_email='mail@pavelsof.com',

    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python :: 3',
        'Topic :: Internet :: WWW/HTTP',
    ],
    keywords='django codemirror',
    project_urls={
        'Source': 'https://github.com/pavelsof/django-mirror',
        'Tracker': 'https://github.com/pavelsof/django-mirror/issues',
    },

    packages=find_packages(exclude=['tests']),

    install_requires=['django>=1.11'],
    python_requires='>=3',

    package_data={
        'django_mirror': [
            'media.json',
            'static/django-mirror/*.css',
            'static/django-mirror/*.js',
            'static/django-mirror/codemirror/addon/*/*.css',
            'static/django-mirror/codemirror/addon/*/*.js',
            'static/django-mirror/codemirror/lib/*.css',
            'static/django-mirror/codemirror/lib/*.js',
            'static/django-mirror/codemirror/mode/*/*.js',
            'static/django-mirror/codemirror/mode/*.js',
            'static/django-mirror/codemirror/theme/*.css',
        ]
    },
)
